local L = LibStub( "AceLocale-3.0" ):GetLocale( "UnderHood" )
local UH = UnderHood
local UnitName = UnitName
local UnitCanAttack = UnitCanAttack
local AceEvent = LibStub( "AceEvent-3.0" )
local AceEvent2 = LibStub( "AceEvent-2.0" )
local Threat = LibStub( "Threat-1.0" )
local _G = _G

-- Centralized threat events distribution

local providers = {}

local function updateThreat(...)
	for provider,_ in pairs( providers ) do
		provider:ThreatUpdated(...)
	end
end

local function attachProvider( provider )
	if not providers[provider] then
		if not next( providers ) then
			AceEvent2:RegisterEvent( "Threat_ThreatUpdated", updateThreat )
		end
		
		providers[provider] = true
	end
end

local function detachProvider( provider )
	providers[provider] = nil
	
	if not next( providers ) then
	    AceEvent2.UnregisterEvent( updateThreat, "Threat_ThreatUpdated")
	end
end

-- Provider

local Provider = {}
Provider.__index = Provider
Provider.name = "Threat"
Provider.title = L["Threat"]
Provider.supportedUnits = { "player" }
Provider.manageVisibility = true

function Provider:Create( bar )
	local self = {}

	setmetatable( self, Provider )
	AceEvent:Embed( self )

	self.bar = bar

	return self
end

function Provider:UnitSupported( unit )
	return unit == "player"
end

function Provider:IsBarVisible()
	return Threat:IsActive() and self.inCombat
end

function Provider:Connect()
	attachProvider( self )
	
	self:RegisterEvent( "PLAYER_TARGET_CHANGED" )

	self:PLAYER_TARGET_CHANGED()
end

function Provider:Disconnect()
	self:UnregisterAllEvents()
	self:UnregisterAllMessages()

	detachProvider( self )
end

function Provider:OnEnterCombat()
	self.inCombat = true
	self.bar:Show()
end

function Provider:OnLeaveCombat()
	self.inCombat = nil
	self.bar:Hide()
end

function Provider:Update()
	local r,g,b = unpack( UH.db.profile.colors.unit.hostile )

	self.bar:SetColor( r, g, b, 1 )

	--[[
	if Threat:IsActive() and self.targetName then
		self.bar:Show()
	else
		self.bar:Hide()
	end
	--]]

	self:SetValue()
end

function Provider:ThreatUpdated( name, unit, hash, value )
	if not( self.targetName and self.inCombat ) then return end

	if unit == "player" or (hash and Threat:UnitIsUnit( self.targetName, hash )) then
	    self:SetValue()
	end
end

function Provider:PLAYER_TARGET_CHANGED()
	self.targetName = nil

	if UnitCanAttack( "player", "target" ) then
	    self.targetName = UnitName( "target" )
	end

	self:Update()
end

function Provider:SetValue()
	local threat, maxThreat

	if Threat:IsActive() and self.targetName then
		threat = Threat:GetThreat( UnitName( "player" ), self.targetName )
		maxThreat = Threat:GetMaxThreatOnTarget( self.targetName )
	end

	if not( maxThreat ) or maxThreat == 0 then
		threat = 0
		maxThreat = 1
	else
		threat = threat/maxThreat
	end

	self.bar:SetValue( threat )
end

UnderHood_Bars:RegisterProvider( Provider )
